@@ -1,10 +1,11 @@
 ///-*-C++-*-//////////////////////////////////////////////////////////////////
 //
-// Hoard: A Fast, Scalable, and Memory-Efficient Allocator
-//        for Shared-Memory Multiprocessors
-// Contact author: Emery Berger, http://www.cs.utexas.edu/users/emery
+// The Hoard Multiprocessor Memory Allocator
+// www.hoard.org
 //
-// Copyright (c) 1998-2000, The University of Texas at Austin.
+// Author: Emery Berger, http://www.cs.utexas.edu/users/emery
+//
+// Copyright (c) 1998-2001, The University of Texas at Austin.
 //
 // This library is free software; you can redistribute it and/or modify
 // it under the terms of the GNU Library General Public License as
@@ -17,7 +18,6 @@
 //
 //////////////////////////////////////////////////////////////////////////////
 
-#include <assert.h>
 
 #include "arch-specific.h"
 
@@ -31,6 +31,12 @@ extern "C" {
 
 #ifdef WIN32
 
+unsigned long hoardInterlockedExchange (unsigned long * oldval,
+					unsigned long newval)
+{
+  return InterlockedExchange (reinterpret_cast<long *>(oldval), newval);
+}
+
 void hoardCreateThread (hoardThreadType& t,
 			void *(*function) (void *),
 			void * arg)
@@ -48,49 +54,79 @@ void hoardSetConcurrency (int)
 }
 
 int hoardGetThreadID (void) {
-  int tid = GetCurrentThreadId();
+  // Windows thread id's are even, so we divide them by two
+  // in order to get a reasonable thread identifier.
+  int tid = GetCurrentThreadId() >> 1;
   return tid;
 }
 
 void hoardLockInit (hoardLockType& mutex) {
-  InterlockedExchange (&mutex, 0);
+	mutex = UNLOCKED;
+	//  InitializeCriticalSection (&mutex);
 }
 
 void hoardLock (hoardLockType& mutex) {
-  // A yielding lock (with an initial spin).
-  int i;
-  while (1) {
-    i = 0;
-    while (i < SPIN_LIMIT) {
-      if (InterlockedExchange (&mutex, LOCKED) == UNLOCKED) {
-	// We got the lock.
-	return;
-      }
-      i++;
+#if 0
+  int spincount = 0;
+  while (InterlockedExchange (&mutex, LOCKED) != UNLOCKED) {
+    spincount++;
+    if (spincount > 100) {
+      hoardYield();
+      spincount = 0;
     }
-    // Yield to other threads.
-    Sleep (1);
   }
+#else
+	while (InterlockedExchange (&mutex, LOCKED) != UNLOCKED) {
+		hoardYield();
+	}
+#endif
+//	EnterCriticalSection (&mutex);
+}
+
+void hoardYield (void) {
+  Sleep (0);
 }
 
 void hoardUnlock (hoardLockType& mutex) {
-  InterlockedExchange (&mutex, UNLOCKED);
+	mutex = UNLOCKED;
+//	LeaveCriticalSection(&mutex);
 }
 
-void * hoardSbrk (long size)
+#if 0
+void hoardLockDestroy (hoardLockType& mutex) {
+//	DeleteCriticalSection(&mutex);
+}
+#endif
+
+static hoardLockType memoryLock = UNLOCKED;
+extern "C" void * dlmalloc (size_t);
+extern "C" void dlfree (void *);
+
+void * hoardGetMemory (long size)
 {
-  void * newMemory = VirtualAlloc (NULL, size, MEM_COMMIT, PAGE_READWRITE);
-  assert (newMemory);
-  return newMemory;
+#if 0
+	hoardLock (memoryLock);
+  void * ptr = dlmalloc (size);
+	hoardUnlock (memoryLock);
+#else
+  void * ptr = HeapAlloc (GetProcessHeap(), 0, size);
+#endif
+
+  return (void *) ptr;
 }
 
 
-void hoardUnsbrk (void * ptr, long size)
+void hoardFreeMemory (void * ptr)
 {
-  VirtualFree (ptr, 0, MEM_RELEASE);
+#if 0
+	hoardLock (memoryLock);
+	dlfree (ptr);
+	hoardUnlock (memoryLock);
+#else
+	HeapFree (GetProcessHeap(), 0, ptr);
+#endif
 }
 
-
 int hoardGetPageSize (void)
 {
   SYSTEM_INFO infoReturn[1];
@@ -101,92 +137,17 @@ int hoardGetPageSize (void)
 
 int hoardGetNumProcessors (void)
 {
-  SYSTEM_INFO infoReturn[1];
-  GetSystemInfo (infoReturn);
-  return (int) (infoReturn -> dwNumberOfProcessors);
+  static int numProcessors = 0;
+  if (numProcessors == 0) {
+    SYSTEM_INFO infoReturn[1];
+    GetSystemInfo (infoReturn);
+    numProcessors = (int) (infoReturn -> dwNumberOfProcessors);
+  }
+  return numProcessors;
 }
 
-#elif defined(__BEOS__)
-
-#include <OS.h>
-#include <unistd.h>
-
-void hoardCreateThread (hoardThreadType &t,
-			void *( *function)( void *),
-			void *arg)
-{
-  t = spawn_thread((int32 (*)(void*))function, "some thread",
-		   B_NORMAL_PRIORITY, arg);
-  if (t >= B_OK) resume_thread(t);
-  else debugger("spawn_thread() failed!");
-}
-
-
-void hoardJoinThread (hoardThreadType &t)
-{
-  status_t dummy;
-  wait_for_thread(t, &dummy);
-}
-
-
-void hoardSetConcurrency (int)
-{
-}
-
-
-int hoardGetThreadID (void)
-{
-  return find_thread(0);
-}
-
-
-void hoardLockInit (hoardLockType &lock)
-{
-  lock.ben = 0;
-  lock.sem = create_sem(0, "a hoard lock");
-}
-
-
-void hoardLock (hoardLockType &lock)
-{
-  if((atomic_add(&(lock.ben), 1)) >= 1) acquire_sem(lock.sem);
-}
-
-
-void hoardUnlock (hoardLockType &lock)
-{
-  if((atomic_add(&(lock.ben), -1)) > 1) release_sem(lock.sem);
-}
-
-int hoardGetPageSize (void)
-{
-  return B_PAGE_SIZE;
-}
-
-int hoardGetNumProcessors (void)
-{
-  system_info si;
-  status_t result = get_system_info(&si);
-  assert (result == B_OK);
-  return si.cpu_count;
-}
-
-
-void * hoardSbrk (long size)
-{
-  return sbrk(size);
-}
-
-
-void hoardUnsbrk (void * ptr, long size)
-{
-  // NOT CURRENTLY IMPLEMENTED!
-}
-
-
 #else // UNIX
 
-
 #if USE_SPROC
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -194,7 +155,6 @@ void hoardUnsbrk (void * ptr, long size)
 #include <ulocks.h>
 #endif
 
-
 void hoardCreateThread (hoardThreadType& t,
 			void *(*function) (void *),
 			void * arg)
@@ -222,10 +182,12 @@ void hoardJoinThread (hoardThreadType& t)
 #endif
 }
 
+#if 0 // DISABLED!
 #if defined(__linux)
-// This extern declaration is required for Linux.
+// This extern declaration is required for some versions of Linux.
 extern "C" void pthread_setconcurrency (int n);
 #endif
+#endif
 
 void hoardSetConcurrency (int n)
 {
@@ -263,9 +225,12 @@ int hoardGetThreadID (void) {
   return pid;
 #else
 #if defined(__linux)
-  // Consecutive thread id's in Linux are 1024 apart;
-  // dividing off the 1024 gives us an appropriate thread id.
-  return (int) pthread_self() >> 10; // (>> 10 = / 1024)
+  // Consecutive thread id's in Linux are 1024 apart.
+  return (int) pthread_self() / 1024;
+#endif
+#if defined(__AIX)
+  // Consecutive thread id's in AIX are 257 apart.
+  return (int) pthread_self() / 257;
 #endif
 #if defined(__SVR4)
   return (int) lwp_self();
@@ -275,12 +240,10 @@ int hoardGetThreadID (void) {
 }
 
 
-// If we are using either Intel or SPARC,
-// we use our own lock implementation
-// (spin then yield). This is much cheaper than
-// the ordinary mutex, at least on Linux and Solaris.
+// Here's our own lock implementation (spin then yield). This is much
+// cheaper than the ordinary mutex, at least on Linux and Solaris.
 
-#if USER_LOCKS && (defined(i386) || defined(sparc) || defined(__sgi) || defined(__ppc))
+#if USER_LOCKS
 
 #include <sched.h>
 
@@ -298,7 +261,7 @@ int hoardGetThreadID (void) {
 extern "C" unsigned long InterlockedExchange (unsigned long * oldval,
 					      unsigned long newval);
 #else
-static inline unsigned long InterlockedExchange (unsigned long * oldval,
+unsigned long InterlockedExchange (unsigned long * oldval,
 						 unsigned long newval)
 {
 #if defined(sparc)
@@ -329,129 +292,88 @@ static inline unsigned long InterlockedExchange (unsigned long * oldval,
 	: "=&r"(ret)
 	: "r"(oldval), "r"(newval), "r"(*oldval)
 	: "cr0", "memory");
+#endif
+#if !(defined(sparc) || defined(i386) || defined(__sgi) || defined(ppc))
+#error "Hoard does not include support for user-level locks for this platform."
 #endif
   return newval;
 }
 #endif
 
+unsigned long hoardInterlockedExchange (unsigned long * oldval,
+					unsigned long newval)
+{
+  return InterlockedExchange (oldval, newval);
+}
+
 void hoardLockInit (hoardLockType& mutex) {
   InterlockedExchange (&mutex, UNLOCKED);
 }
 
+#include <stdio.h>
+
 void hoardLock (hoardLockType& mutex) {
-  // A yielding lock (with an initial spin).
-  int i;
-  while (1) {
-    i = 0;
-    while (i < SPIN_LIMIT) {
-      if (InterlockedExchange (&mutex, LOCKED) == UNLOCKED) {
-	// We got the lock.
-	return;
-      }
-      i++;
+  int spincount = 0;
+  while (InterlockedExchange (&mutex, LOCKED) != UNLOCKED) {
+    spincount++;
+    if (spincount > 100) {
+      hoardYield();
+      spincount = 0;
     }
-    // The lock is still being held by someone else.
-    // Give up our quantum.
-    sched_yield ();
   }
 }
 
 void hoardUnlock (hoardLockType& mutex) {
-  InterlockedExchange (&mutex, UNLOCKED);
+	mutex = UNLOCKED;
+//  InterlockedExchange (&mutex, UNLOCKED);
 }
 
-#else // use non-user-level locks. 
+#else
 
-void hoardLockInit (hoardLockType& lock) {
-  pthread_mutex_init (&lock, NULL);
-}
-
-void hoardLock (hoardLockType& lock) {
-  pthread_mutex_lock (&lock);
-}
-
-void hoardUnlock (hoardLockType& lock) {
-  pthread_mutex_unlock (&lock);
-}
+// use non-user-level locks. 
 
 #endif // USER_LOCKS
 
 
-#if !(defined(__sgi))
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <sys/mman.h>
+#if defined(__SVR4)
+#include <thread.h>
 #endif
 
-
-void * hoardSbrk (long size) {
-  // Use mmap for everything but SGI.
-  // Apparently sproc's and mmap's don't cooperate.
-#if defined(__sgi)
-  // Protect calls to sbrk() with a lock.
-#if USER_LOCKS
-  static hoardLockType sbrkLock = 0;
+void hoardYield (void)
+{
+#if defined(__SVR4)
+  thr_yield();
 #else
-  static hoardLockType sbrkLock = PTHREAD_MUTEX_INITIALIZER;
-#endif
-
-  hoardLock (sbrkLock);
-  void * moreMemory = sbrk (size);
-  hoardUnlock (sbrkLock);
-  return moreMemory;
-#else
-
-#if defined(linux)
-#if USER_LOCKS
-  static hoardLockType sbrkLock = 0;
-#else
-  static hoardLockType sbrkLock = PTHREAD_MUTEX_INITIALIZER;
-#endif
-#endif
-  // Use mmap to get memory from the backing store.
-  static int fd = ::open ("/dev/zero", O_RDWR);
-#if defined(linux)
-  hoardLock (sbrkLock);
-#endif
-  char * ptr = (char *) mmap (NULL, size, PROT_READ | PROT_WRITE | PROT_EXEC, MAP_PRIVATE, fd, 0);
-  char * retPtr;
-  if (ptr == (char *) MAP_FAILED) {
-    retPtr = NULL;
-  } else {
-    retPtr = ptr;
-  }
-#if defined(linux)
-  hoardUnlock (sbrkLock);
-#endif
-  return retPtr;
+  sched_yield();
 #endif
 }
 
 
-void hoardUnsbrk (void * ptr, long size)
-{
-#if defined(__sgi)
-  // Do nothing.
-#else
-  // Unmap the region.
-#if defined(linux)
+extern "C" void * dlmalloc (size_t);
+extern "C" void dlfree (void *);
+
+
 #if USER_LOCKS
-  static hoardLockType sbrkLock = 0;
+static hoardLockType getMemoryLock = 0;
 #else
-  static hoardLockType sbrkLock = PTHREAD_MUTEX_INITIALIZER;
-#endif
-#endif
-#if defined(linux)
-  hoardLock (sbrkLock);
-#endif
-  int result = munmap ((char *) ptr, size);
-  assert (result == 0);
-#if defined(linux)
-  hoardUnlock (sbrkLock);
-#endif
+static hoardLockType getMemoryLock = PTHREAD_MUTEX_INITIALIZER;
 #endif
+
+#include <stdio.h>
+
+void * hoardGetMemory (long size) {
+  hoardLock (getMemoryLock);
+  void * ptr = dlmalloc (size);
+  hoardUnlock (getMemoryLock);
+  return ptr;
+}
+
+
+void hoardFreeMemory (void * ptr)
+{
+  hoardLock (getMemoryLock);
+  dlfree (ptr);
+  hoardUnlock (getMemoryLock);
 }
 
 
@@ -478,33 +400,39 @@ int hoardGetNumProcessors (void)
 {
 #if !(defined(linux))
 #if defined(__sgi)
-  return (int) sysmp(MP_NAPROCS);
+  static int np = (int) sysmp(MP_NAPROCS);
+  return np;
 #else
-  return (int) sysconf(_SC_NPROCESSORS_ONLN);
+  static int np = (int) sysconf(_SC_NPROCESSORS_ONLN);
+  return np;
 #endif
 #else
-  // Ugly workaround.  Linux's sysconf indirectly calls malloc() (at
-  // least on multiprocessors).  So we just read the info from the
-  // proc file ourselves and count the occurrences of the word
-  // "processor".
+  static int numProcessors = 0;
 
-  // We only parse the first 32K of the CPU file.  By my estimates,
-  // that should be more than enough for at least 64 processors.
-  enum { MAX_PROCFILE_SIZE = 32768 };
-  char line[MAX_PROCFILE_SIZE];
-  int numProcessors = 0;
-  int fd = open ("/proc/cpuinfo", O_RDONLY);
-  assert (fd);
-  read(fd, line, MAX_PROCFILE_SIZE);
-  char * str = line;
-  while (str) {
-    str = strstr(str, "processor");
-    if (str) {
-      numProcessors++;
-      str++;
+  if (numProcessors == 0) {
+    // Ugly workaround.  Linux's sysconf indirectly calls malloc() (at
+    // least on multiprocessors).  So we just read the info from the
+    // proc file ourselves and count the occurrences of the word
+    // "processor".
+    
+    // We only parse the first 32K of the CPU file.  By my estimates,
+    // that should be more than enough for at least 64 processors.
+    enum { MAX_PROCFILE_SIZE = 32768 };
+    char line[MAX_PROCFILE_SIZE];
+    int fd = open ("/proc/cpuinfo", O_RDONLY);
+    //    assert (fd);
+    read(fd, line, MAX_PROCFILE_SIZE);
+    char * str = line;
+    while (str) {
+      str = strstr(str, "processor");
+      if (str) {
+	numProcessors++;
+	str++;
+      }
     }
+    close (fd);
+    //    assert (numProcessors > 0);
   }
-  close (fd);
   return numProcessors;
 #endif
 }
