@@ -1,10 +1,11 @@
 ///-*-C++-*-//////////////////////////////////////////////////////////////////
 //
-// Hoard: A Fast, Scalable, and Memory-Efficient Allocator
-//        for Shared-Memory Multiprocessors
-// Contact author: Emery Berger, http://www.cs.utexas.edu/users/emery
+// The Hoard Multiprocessor Memory Allocator
+// www.hoard.org
 //
-// Copyright (c) 1998-2000, The University of Texas at Austin.
+// Author: Emery Berger, http://www.cs.utexas.edu/users/emery
+//
+// Copyright (c) 1998-2001, The University of Texas at Austin.
 //
 // This library is free software; you can redistribute it and/or modify
 // it under the terms of the GNU Library General Public License as
@@ -31,18 +32,17 @@
 #ifndef WIN32
 #define WIN32 1
 #endif
+
+// Set maximal inlining.
+#pragma inline_depth(255)
+#define inline __forceinline
+
 #include <windows.h>
 #include <process.h>
-typedef LONG	hoardLockType;
+//typedef CRITICAL_SECTION	hoardLockType;
+typedef long hoardLockType;
 typedef HANDLE 	hoardThreadType;
 
-#elif defined(__BEOS__)
-
-// BeOS
-
-#include <OS.h>
-
-
 #elif USE_SPROC
 
 // SGI's SPROC library
@@ -65,12 +65,7 @@ typedef HANDLE 	hoardThreadType;
 
 
 #ifndef WIN32
-#if defined(__BEOS__)
-typedef struct {
-  int32 ben;
-  sem_id sem;
-} hoardLockType;
-#elif USER_LOCKS && (defined(i386) || defined(sparc) || defined(__sgi))
+#if USER_LOCKS && (defined(i386) || defined(sparc) || defined(__sgi))
 typedef unsigned long	hoardLockType;
 #else
 typedef pthread_mutex_t	hoardLockType;
@@ -78,8 +73,6 @@ typedef pthread_mutex_t	hoardLockType;
 
 #if USE_SPROC
 typedef pid_t			hoardThreadType;
-#elif defined(__BEOS__)
-typedef thread_id               hoardThreadType;
 #else
 typedef pthread_t		hoardThreadType;
 #endif
@@ -103,20 +96,43 @@ extern "C" {
 
   ///// Lock-related wrappers.
 
+#if !defined(WIN32) && !USER_LOCKS
+
+  // Define the lock operations inline to save a little overhead.
+
+  inline void hoardLockInit (hoardLockType& lock) {
+    pthread_mutex_init (&lock, NULL);
+  }
+
+  inline void hoardLock (hoardLockType& lock) {
+    pthread_mutex_lock (&lock);
+  }
+
+  inline void hoardUnlock (hoardLockType& lock) {
+    pthread_mutex_unlock (&lock);
+  }
+
+#else
   void	hoardLockInit (hoardLockType& lock);
   void	hoardLock (hoardLockType& lock);
   void	hoardUnlock (hoardLockType& lock);
+#endif
+
+  inline void  hoardLockDestroy (hoardLockType&) {
+  }
 
   ///// Memory-related wrapper.
 
   int	hoardGetPageSize (void);
-  void *	hoardSbrk (long size);
-  void	hoardUnsbrk (void * ptr, long size);
+  void *	hoardGetMemory (long size);
+  void	hoardFreeMemory (void * ptr);
 
   ///// Other.
 
+  void  hoardYield (void);
   int	hoardGetNumProcessors (void);
-
+  unsigned long hoardInterlockedExchange (unsigned long * oldval,
+					  unsigned long newval);
 }
 
 #endif // _ARCH_SPECIFIC_H_
