@@ -1,10 +1,11 @@
 ///-*-C++-*-//////////////////////////////////////////////////////////////////
 //
-// Hoard: A Fast, Scalable, and Memory-Efficient Allocator
-//        for Shared-Memory Multiprocessors
-// Contact author: Emery Berger, http://www.cs.utexas.edu/users/emery
+// The Hoard Multiprocessor Memory Allocator
+// www.hoard.org
 //
-// Copyright (c) 1998-2000, The University of Texas at Austin.
+// Author: Emery Berger, http://www.cs.utexas.edu/users/emery
+//
+// Copyright (c) 1998-2001, The University of Texas at Austin.
 //
 // This library is free software; you can redistribute it and/or modify
 // it under the terms of the GNU Library General Public License as
@@ -35,7 +36,7 @@ public:
 #if HEAP_DEBUG
       _magic (FREE_BLOCK_MAGIC),
 #endif
-      _next (0),
+      _next (NULL),
       _mySuperblock (sb)
     {}
 
@@ -70,43 +71,17 @@ public:
 #if HEAP_FRAG_STATS
   void setRequestedSize (size_t s) 
   {   
-    // cout << "Setting requested size = " << s << endl;
     _requestedSize = s;
   }
 
   size_t getRequestedSize (void) { return _requestedSize; }
 #endif
 
-#if USE_PRIVATE_HEAPS
-  void setActualSize (size_t s) { _actualSize = s; }
-  size_t getActualSize (void) { return _actualSize; }
-#endif
-
   void setNext (block * b) { _next = b; }
   block * getNext (void) { return _next; }
 
 private:
 
-#if USE_PRIVATE_HEAPS
-
-#if HEAP_DEBUG
-  union {
-    unsigned long _magic;
-    double _d1; // For alignment.
-  };
-#endif
-
-  block * _next;	// The next block in a linked-list of blocks.
-  size_t _actualSize;	// The actual size of the block.
-
-  union {
-    double _d2; // For alignment.
-    superblock *	_mySuperblock;	// A pointer to my superblock.
-  };
-
-
-#else // ! USE_PRIVATE_HEAPS
-
 #if HEAP_DEBUG
   union {
     unsigned long _magic;
@@ -117,8 +92,6 @@ private:
   block *	_next;		// The next block in a linked-list of blocks.
   superblock *	_mySuperblock;	// A pointer to my superblock.
 
-#endif // USE_PRIVATE_HEAPS
-
 #if HEAP_FRAG_STATS
   union {
     double _d4; // This is just for alignment purposes.
