@@ -1,10 +1,11 @@
 ///-*-C++-*-//////////////////////////////////////////////////////////////////
 //
-// Hoard: A Fast, Scalable, and Memory-Efficient Allocator
-//        for Shared-Memory Multiprocessors
-// Contact author: Emery Berger, http://www.cs.utexas.edu/users/emery
+// The Hoard Multiprocessor Memory Allocator
+// www.hoard.org
 //
-// Copyright (c) 1998-2000, The University of Texas at Austin.
+// Author: Emery Berger, http://www.cs.utexas.edu/users/emery
+//
+// Copyright (c) 1998-2001, The University of Texas at Austin.
 //
 // This library is free software; you can redistribute it and/or modify
 // it under the terms of the GNU Library General Public License as
@@ -16,6 +17,7 @@
 // Library General Public License for more details.
 //
 //////////////////////////////////////////////////////////////////////////////
+
 #ifndef _CONFIG_H_
 #define _CONFIG_H_
 
@@ -23,38 +25,54 @@
 #define _REENTRANT		// If defined, generate a multithreaded-capable version.
 #endif
 
-// The logarithmic base used for size classes.
-// (i.e., an object is in size class i if
-//        base^(i-1) * ALIGNMENT < size <= base^i * ALIGNMENT.)
-// Note that this puts an upper-limit on internal fragmentation:
-//   if SIZE_CLASS_BASE is 1.2, then we will never see more than
-//   20% internal fragmentation (for aligned requests).
+#ifndef USER_LOCKS
+#if !(defined(sparc) || defined(i386) || defined(__sgi) || defined(ppc))
+#define USER_LOCKS 0
+#else
+#define USER_LOCKS 1		// Use our own user-level locks if they're available for the current architecture.
+#endif
+#endif
 
-#define SIZE_CLASS_BASE 1.2
+#define HEAP_LOG 0		// If non-zero, keep a log of heap accesses.
+
+
+///// You should not change anything below here. /////
+
+#if defined(WIN32)
+
+#define _CRTIMP __declspec(dllimport)
+
+#if 0
+#ifndef __PLACEMENT_NEW_INLINE
+#define __PLACEMENT_NEW_INLINE
+inline void *__cdecl operator new(size_t, void *_P) { return (_P); }
+#if _MSC_VER >= 1200
+inline void __cdecl operator delete(void *, void *) { return; }
+#endif
+#endif
+#endif
+
+#endif
 
 // The number of groups of superblocks we maintain based on what
 // fraction of the superblock is empty. NB: This number must be at
-// least 2.
+// least 2, and is 1 greater than the EMPTY_FRACTION in heap.h.
 
-enum { SUPERBLOCK_FULLNESS_GROUP = 3 };
+enum { SUPERBLOCK_FULLNESS_GROUP = 9 };
 
 
-#if 0
-#define DEBUG_PRINT printf
-#else
-#define DEBUG_PRINT dummy_printf
-#endif
+
+// DO NOT CHANGE THESE.  They require running of maketable to replace
+// the values in heap.cpp for the _numBlocks array.
 
 #define HEAP_DEBUG 0		// If non-zero, keeps extra info for sanity checking.
 #define HEAP_STATS 0		// If non-zero, maintain blowup statistics.
-#define HEAP_FRAG_STATS 0		// If non-zero, maintain fragmentation statistics.
-#define HEAP_LOG 0		// If non-zero, keep a log of heap accesses.
+#define HEAP_FRAG_STATS 0	// If non-zero, maintain fragmentation statistics.
 
-#define USER_LOCKS 1		// Use our own user-level locks if they're available for the current architecture.
 
 // CACHE_LINE = The number of bytes in a cache line.
 
-#ifdef i386
+#if defined(i386) || defined(WIN32)
 #define CACHE_LINE 32
 #endif
 
@@ -72,12 +90,6 @@ enum { SUPERBLOCK_FULLNESS_GROUP = 3 };
 #define CACHE_LINE 64
 #endif
 
-// dummy_printf():
-//   a stub that is used when we disable HEAP_DEBUG
-//   so that nothing is printed.
-
-inline void dummy_printf (...) {}
-
 #ifdef __GNUG__
 // Use the max operator, an extension to C++ found in GNU C++.
 #define MAX(a,b) ((a) >? (b))
