@@ -75,7 +75,6 @@ private:
 
 inline void heapStats::incStats (int updateU, int updateA) 
 {
-  DEBUG_PRINT ("incstats: %d, %d\n", updateU, updateA);
   assert (updateU >= 0);
   assert (updateA >= 0);
   assert (U <= A);
@@ -109,11 +108,11 @@ inline void heapStats::incUStats (void)
 
 inline void heapStats::decStats (int updateU, int updateA)
 {
-  DEBUG_PRINT ("decstats: %d, %d\n", updateU, updateA);
   assert (updateU >= 0);
   assert (updateA >= 0);
   assert (U <= A);
-  assert (A >= 0);
+  assert (U >= updateU);
+  assert (A >= updateA);
   U -= updateU;
   A -= updateA;
   assert (U <= A);
