@@ -1,10 +1,11 @@
 ///-*-C++-*-//////////////////////////////////////////////////////////////////
 //
-// Hoard: A Fast, Scalable, and Memory-Efficient Allocator
-//        for Shared-Memory Multiprocessors
-// Contact author: Emery Berger, http://www.cs.utexas.edu/users/emery
+// The Hoard Multiprocessor Memory Allocator
+// www.hoard.org
 //
-// Copyright (c) 1998-2000, The University of Texas at Austin.
+// Author: Emery Berger, http://www.cs.utexas.edu/users/emery
+//
+// Copyright (c) 1998-2001, The University of Texas at Austin.
 //
 // This library is free software; you can redistribute it and/or modify
 // it under the terms of the GNU Library General Public License as
@@ -17,35 +18,24 @@
 //
 //////////////////////////////////////////////////////////////////////////////
 
-#include <string.h>
-#include <stdio.h>
-
 #include "config.h"
 
-#if USE_PRIVATE_HEAPS
-#include "privateheap.h"
-#define HEAPTYPE privateHeap
-#else
-#define HEAPTYPE threadHeap
 #include "threadheap.h"
-#endif
 
 #include "processheap.h"
 
-hoardLockType processHeap::_bufferLock;
-superblock * processHeap::_recycleBin;
-
 
 processHeap::processHeap (void)
+  : _buffer (NULL),
+    _bufferCount (0)
 #if HEAP_FRAG_STATS
-  : _currentAllocated (0),
+  , _currentAllocated (0),
   _currentRequested (0),
   _maxAllocated (0),
   _inUseAtMaxAllocated (0),
   _maxRequested (0)
 #endif
 {
-  _recycleBin = NULL;
   int i;
   // The process heap is heap 0.
   setIndex (0);
@@ -70,46 +60,6 @@ processHeap::processHeap (void)
 }
 
 
-// Obtain a superblock buffer from our store.  We always ask the
-// system for at least the REFILL number of superblocks (above), and
-// then parcel them out to the threads.
-// Returns 1 if this is a recycled superblock.
-int processHeap::getSuperblockBuffer (char *& ptr)
-{
-  static char * superblocksBuffer = NULL;
-  static int superblocksBufferCount = 0;
-
-  hoardLock (_bufferLock);
-
-  // If we're out of buffer space,
-  // refill with a chunk that can hold the refill number of superblocks
-  // if there are none available in the recycle bin.
-
-  int isRecycled = 0;
-  if (_recycleBin) {
-    // Reuse a superblock.
-    ptr = (char *) _recycleBin;
-    ((superblock *) ptr)->remove();
-    _recycleBin = _recycleBin->getNext();
-    isRecycled = 1;
-  } else {
-    if (superblocksBufferCount == 0) {
-      // Refill the superblock buffer.
-      superblocksBuffer
-	= (char *) hoardSbrk (SUPERBLOCK_SIZE * REFILL_NUMBER_OF_SUPERBLOCKS);
-      superblocksBufferCount = REFILL_NUMBER_OF_SUPERBLOCKS;
-    }
-    // Carve out a superblock and return it.
-    ptr = superblocksBuffer;
-    superblocksBuffer += SUPERBLOCK_SIZE;
-    superblocksBufferCount--;
-  }
-
-  hoardUnlock (_bufferLock);
-  return isRecycled;
-}
-
-
 // Print out statistics information.
 void processHeap::stats (void) {
 #if HEAP_STATS
@@ -117,7 +67,6 @@ void processHeap::stats (void) {
   int amax = 0;
   for (int j = 0; j < MAX_HEAPS; j++) {
     for (int i = 0; i < SIZE_CLASSES; i++) {
-      // printf ("[%d] U=%d, A=%d\n", i, theap[j].maxInUse(i), theap[j].maxAllocated(i));
       amax += theap[j].maxAllocated(i) * sizeFromClass (i);
       umax += theap[j].maxInUse(i) * sizeFromClass (i);
     }
@@ -132,7 +81,14 @@ void processHeap::stats (void) {
   printf ("Fragmentation (4) = %f\n", (float) amax / (float) umax);
 #endif
 
-#endif // HEAP_STATS
+#endif // HEAP_STATS  
+#if HEAP_LOG
+  printf ("closing logs.\n");
+  fflush (stdout);
+  for (int i = 0; i < MAX_HEAPS + 1; i++) {
+    _log[i].close();
+  }
+#endif
 }
 
 
@@ -142,7 +98,6 @@ void processHeap::setAllocated (int requestedSize,
 				int actualSize)
 {
   hoardLock (_statsLock);
-  // printf ("setAllocated (%d, %d)\n", requestedSize, actualSize);
   _currentRequested += requestedSize;
   _currentAllocated += actualSize;
   if (_currentRequested > _maxRequested) {
@@ -188,7 +143,8 @@ void processHeap::free (void * ptr)
   assert (b->isValid());
 
   // Check to see if this block came from a memalign() call.
-  if (((unsigned long) b->getNext() & 1) == 1) {
+  if ((unsigned long) b->getNext() & 1) {
+    // It did. Set the block to the actual block header.
     b = (block *) ((unsigned long) b->getNext() & ~1);
     assert (b->isValid());
   }    
@@ -199,10 +155,6 @@ void processHeap::free (void * ptr)
   assert (sb);
   assert (sb->isValid());
 
-#if HEAP_FRAG_STATS
-  setDeallocated (b->getRequestedSize(), 0);
-#endif
-
   const int sizeclass = sb->getBlockSizeClass();
 
   //
@@ -211,14 +163,39 @@ void processHeap::free (void * ptr)
   // and update its statistics.
   //
 
-  hoardHeap * owner = getAndLockOwner (sb);
+  hoardHeap * owner;
+
+  // By acquiring the up lock on the superblock,
+  // we prevent it from moving to the global heap.
+  // This eventually pins it down in one heap,
+  // so this loop is guaranteed to terminate.
+  // (It should generally take no more than two iterations.)
+  sb->upLock();
+  for (;;) {
+    owner = sb->getOwner();
+    owner->lock();
+    if (owner == sb->getOwner()) {
+      break;
+    } else {
+      owner->unlock();
+    }
+    // Suspend to allow ownership to quiesce.
+    hoardYield();
+  }
 
 #if HEAP_LOG
   MemoryRequest m;
   m.free (ptr);
-  getLog (getIndex()).append(m);
+  getLog (owner->getIndex()).append(m);
+#endif
+#if HEAP_FRAG_STATS
+  setDeallocated (b->getRequestedSize(), 0);
 #endif
 
-  owner->freeBlock (b, sb, sizeclass, this);
+  int sbUnmapped = owner->freeBlock (b, sb, sizeclass, this);
+
   owner->unlock();
+  if (!sbUnmapped) {
+    sb->upUnlock();
+  }
 }
