@@ -1,10 +1,11 @@
 ///-*-C++-*-//////////////////////////////////////////////////////////////////
 //
-// Hoard: A Fast, Scalable, and Memory-Efficient Allocator
-//        for Shared-Memory Multiprocessors
-// Contact author: Emery Berger, http://www.cs.utexas.edu/users/emery
+// The Hoard Multiprocessor Memory Allocator
+// www.hoard.org
 //
-// Copyright (c) 1998-2000, The University of Texas at Austin.
+// Author: Emery Berger, http://www.cs.utexas.edu/users/emery
+//
+// Copyright (c) 1998-2001, The University of Texas at Austin.
 //
 // This library is free software; you can redistribute it and/or modify
 // it under the terms of the GNU Library General Public License as
@@ -22,7 +23,7 @@
   ------------------------------------------------------------------------
   We use one processHeap for the whole program.
   ------------------------------------------------------------------------
-  @(#) $Id: processheap.h,v 1.51 2000/02/12 21:45:53 emery Exp $
+  @(#) $Id: processheap.h,v 1.84 2001/12/20 15:36:19 emery Exp $
   ------------------------------------------------------------------------
   Emery Berger                    | <http://www.cs.utexas.edu/users/emery>
   Department of Computer Sciences |             <http://www.cs.utexas.edu>
@@ -36,17 +37,10 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <stdlib.h>
 
 #include "arch-specific.h"
-#include "heap.h"
-#if USE_PRIVATE_HEAPS
-#include "privateheap.h"
-#define HEAPTYPE privateHeap
-#else
-#define HEAPTYPE threadHeap
+#include "hoardheap.h"
 #include "threadheap.h"
-#endif
 
 #if HEAP_LOG
 #include "memstat.h"
@@ -66,11 +60,6 @@ public:
   ~processHeap (void) {
 #if HEAP_STATS
     stats();
-#endif
-#if HEAP_LOG
-    for (int i = 0; i < MAX_HEAPS + 1; i++) {
-      _log[i].close();
-    }
 #endif
   }
 
@@ -84,23 +73,15 @@ public:
   inline int getHeapIndex (void);
 
   // Get the thread heap with index i.
-  inline HEAPTYPE& getHeap (int i);
+  inline threadHeap& getHeap (int i);
 
   // Extract a superblock.
-  inline superblock * acquire (const int c, HEAPTYPE * to);
+  inline superblock * acquire (const int c,
+			       hoardHeap * dest);
 
   // Insert a superblock.
   inline void release (superblock * sb);
 
-  // Obtain a superblock buffer from our store.  We always ask the
-  // system for at least the REFILL number of superblocks (above), and
-  // then parcel them out to the threads.
-  // Returns 1 if this is a recycled superblock.
-  static int getSuperblockBuffer (char *&);
-
-  // Add an empty superblock to the recycle list.
-  inline void recycle (superblock * sb);
-
 #if HEAP_LOG
   // Get the log for index i.
   inline Log<MemoryRequest>& getLog (int i);
@@ -135,17 +116,28 @@ public:
 
 private:
 
-  // Get and lock the owner of a given superblock.
-  inline hoardHeap * getAndLockOwner (superblock *);
+  // Hide the lock & unlock methods.
+
+  inline void lock (void) {
+    hoardHeap::lock();
+  }
+
+  inline void unlock (void) {
+    hoardHeap::unlock();
+  }
 
   // Prevent copying and assignment.
   processHeap (const processHeap&);
   const processHeap& operator= (const processHeap&);
 
   // The per-thread heaps.
-  HEAPTYPE theap[MAX_HEAPS];
+  threadHeap theap[MAX_HEAPS];
 
 #if HEAP_FRAG_STATS
+  // Statistics required to compute fragmentation.  We cannot
+  // unintrusively keep track of these on a multiprocessor, because
+  // this would become a bottleneck.
+
   int _currentAllocated;
   int _currentRequested;
   int _maxAllocated;
@@ -153,22 +145,23 @@ private:
   int _inUseAtMaxAllocated;
   int _fragmentation;
 
-  hoardLockType _statsLock;		// A lock to protect these statistics.
+  // A lock to protect these statistics.
+  hoardLockType _statsLock;
 #endif
 
 #if HEAP_LOG
   Log<MemoryRequest> _log[MAX_HEAPS + 1];
 #endif
 
-  // A lock for the buffer of superblocks.
-  static hoardLockType _bufferLock;
+  // A lock for the superblock buffer.
+  hoardLockType _bufferLock;
 
-  // Completely empty superblocks.
-  static superblock * _recycleBin;
+  char * 	_buffer;
+  int 		_bufferCount;
 };
 
 
-HEAPTYPE& processHeap::getHeap (int i)
+threadHeap& processHeap::getHeap (int i)
 {
   assert (i >= 0);
   assert (i < MAX_HEAPS);
@@ -204,40 +197,23 @@ static int lg (int num)
 
 // Hash out the thread id to a heap and return an index to that heap.
 int processHeap::getHeapIndex (void) {
-  // Here we use the number of processors as the maximum number of heaps.
-  // In fact, for efficiency, we just round up to the highest power of two,
-  // times two.
-  static const int numProcessors = hoardGetNumProcessors();
-  static const int numProcessorsMask = (1 << (lg(numProcessors) + 1)) - 1;
-
-  int tid = hoardGetThreadID() & numProcessorsMask;
+//  int tid = hoardGetThreadID() & hoardHeap::_numProcessorsMask;
+  int tid = hoardGetThreadID() & MAX_HEAPS_MASK;
   assert (tid < MAX_HEAPS);
-
   return tid;
 }
 
 
 superblock * processHeap::acquire (const int sizeclass,
-				   HEAPTYPE * to)
+				   hoardHeap * dest)
 {
-
   lock ();
 
-  // This removes the warning when assertions are turned off.
-  to = to;
-
-  // We can't acquire a superblock from ourselves.
-  assert ((hoardHeap *) to != (hoardHeap *) this);
-
   // Remove the superblock with the most free space.
   superblock * maxSb = removeMaxSuperblock (sizeclass);
-
-#if HEAP_FRAG_STATS
-  // Pretend the released superblock was allocated.
   if (maxSb) {
-    // setAllocated (0, maxSb->getNumBlocks() * sizeFromClass(maxSb->getBlockSizeClass()));
+    maxSb->setOwner (dest);
   }
-#endif
 
   unlock ();
 
@@ -250,47 +226,12 @@ void processHeap::release (superblock * sb)
 {
   assert (EMPTY_FRACTION * sb->getNumAvailable() > sb->getNumBlocks());
 
+  lock();
+
   // Insert the superblock.
-  insertSuperblock (sb->getBlockSizeClass(), sb);
+  insertSuperblock (sb->getBlockSizeClass(), sb, this);
 
-#if HEAP_FRAG_STATS
-  // Pretend the released superblock was de-allocated.
-  // setDeallocated (0, sb->getNumBlocks() * sizeFromClass(sb->getBlockSizeClass()));
-#endif
-}
-
-
-hoardHeap * processHeap::getAndLockOwner (superblock * sb)
-{
-  // Lock only the owner of this superblock.
-  // Since the owner might change between executing
-  // getOwner() and locking the owner's heap,
-  // we need this loop to make sure we lock the right heap.
-
-  hoardHeap * owner;
-  while (1) {
-    owner = sb->getOwner();
-    if (owner) {
-      owner->lock ();
-      if (sb->getOwner() != owner) {
-	// The superblock moved;
-	// we'll have to try again.
-	owner->unlock ();
-      } else {
-	// Success!
-	break;
-      }
-    }
-  }
-  return owner;
-}
-
-
-void processHeap::recycle (superblock * sb)
-{
-  assert (sb->getNumAvailable() == sb->getNumBlocks());
-  sb->insertBefore (_recycleBin);
-  _recycleBin = sb;
+  unlock();
 }
 
 
