@@ -1,10 +1,11 @@
 ///-*-C++-*-//////////////////////////////////////////////////////////////////
 //
-// Hoard: A Fast, Scalable, and Memory-Efficient Allocator
-//        for Shared-Memory Multiprocessors
-// Contact author: Emery Berger, http://www.cs.utexas.edu/users/emery
+// The Hoard Multiprocessor Memory Allocator
+// www.hoard.org
 //
-// Copyright (c) 1998-2000, The University of Texas at Austin.
+// Author: Emery Berger, http://www.cs.utexas.edu/users/emery
+//
+// Copyright (c) 1998-2001, The University of Texas at Austin.
 //
 // This library is free software; you can redistribute it and/or modify
 // it under the terms of the GNU Library General Public License as
@@ -23,7 +24,7 @@
   The superblock class controls a number of blocks (which are
   allocatable units of memory).
   ------------------------------------------------------------------------
-  @(#) $Id: superblock.cpp,v 1.25 2000/02/24 03:11:03 emery Exp $
+  @(#) $Id: superblock.cpp,v 1.45 2001/12/18 02:42:56 emery Exp $
   ------------------------------------------------------------------------
   Emery Berger                    | <http://www.cs.utexas.edu/users/emery>
   Department of Computer Sciences |             <http://www.cs.utexas.edu>
@@ -36,7 +37,7 @@
 
 #include "arch-specific.h"
 #include "config.h"
-#include "heap.h"
+#include "hoardheap.h"
 #include "processheap.h"
 #include "superblock.h"
 
@@ -55,7 +56,8 @@ superblock::superblock (int numBlocks,	// The number of blocks in the sb.
     _freeList (NULL),
     _owner (o),
     _next (NULL),
-    _prev (NULL)
+    _prev (NULL),
+    dirtyFullness (true)
 {
   assert (_numBlocks >= 1);
 
@@ -82,52 +84,45 @@ superblock::superblock (int numBlocks,	// The number of blocks in the sb.
     b = (block *) ((char *) b + blksize);
   }
   _numAvailable = _numBlocks;
-  _fullness = computeFullness();
+  //  computeFullness();
   assert ((unsigned long) b <= hoardHeap::align (sizeof(superblock) + blksize * _numBlocks) + (unsigned long) this);
+
+  hoardLockInit (_upLock);
 }
 
 
-superblock * superblock::makeSuperblock (int sizeclass)
+superblock * superblock::makeSuperblock (int sizeclass,
+					 processHeap * pHeap)
 {
-  // Determine how many blocks will go in this superblock.
-  const size_t s = hoardHeap::sizeFromClass (sizeclass);
-  assert (s > 0);
+  // We need to get more memory.
 
-  // Compute the aligned size of a block plus its header.
-  const int blksize =
-    hoardHeap::align (sizeof(block) + s);
-
-  // Compute the number of blocks that will go into this superblock.
-  int numBlocks = MAX (1, ((hoardHeap::SUPERBLOCK_SIZE - sizeof(superblock)) / blksize));
-
-  // We need to extend the break value to get more memory.
+  char * buf;
+  int numBlocks = hoardHeap::numBlocks(sizeclass);
 
   // Compute how much memory we need.
   unsigned long moreMemory;
+  size_t sz = hoardHeap::sizeFromClass(sizeclass);
+
   if (numBlocks > 1) {
+	// hoardHeap::align(sizeof(superblock) + (hoardHeap::align (sizeof(block) + sz)) * numBlocks) <= hoardHeap::SUPERBLOCK_SIZE) {
+
     moreMemory = hoardHeap::SUPERBLOCK_SIZE;
-    assert (moreMemory >= hoardHeap::align(sizeof(superblock) + blksize * numBlocks));
+    assert (moreMemory >= hoardHeap::align(sizeof(superblock) + (hoardHeap::align (sizeof(block) + sz)) * numBlocks));
+
+    // Get some memory from the process heap.
+    buf = (char *) hoardGetMemory (moreMemory);
+
   } else {
-    moreMemory =
-      hoardHeap::align (sizeof(superblock)  //   one superblock (header)
-			+ blksize * numBlocks);	// + n blocks (headers + data).
+    // One object.
+    assert (numBlocks == 1);
+
+    size_t blksize = hoardHeap::align (sizeof(block) + sz);
+    moreMemory = hoardHeap::align (sizeof(superblock) + blksize);
+
+    // Get space from the system.
+    buf = (char *) hoardGetMemory (moreMemory);
   }
-
-  // Set aside some memory for the new superblock.
-  char * buf;
-
-  // Is the block we obtain a recycled superblock?
-  int isRecycled = 0;
-
-  // Allocate more memory (plus enough to guarantee alignment).
-  if (numBlocks > 1) {
-    // This is an ordinary superblock.
-    // Grab one from the process heap.
-    isRecycled = processHeap::getSuperblockBuffer (buf);
-  } else {
-    buf = (char *) hoardSbrk (moreMemory + hoardHeap::ALIGNMENT - 1);
-  }
-  
+ 
   // Make sure that we actually got the memory.
   if (buf == NULL) {
     return 0;
@@ -138,14 +133,8 @@ superblock * superblock::makeSuperblock (int sizeclass)
   assert (buf == (char *) hoardHeap::align ((unsigned long) buf));
   assert ((((unsigned long) buf) & hoardHeap::ALIGNMENT_MASK) == 0);
 
-  // If the superblock is recycled, check to see if it's for the same
-  // size class.  If so, we don't have to reformat it.
-  if (isRecycled) {
-    if (((superblock *) buf)->getBlockSizeClass() == sizeclass) {
-      return (superblock *) buf;
-    }
-  }
   // Instantiate the new superblock in the buffer.
   superblock * sb = new (buf) superblock (numBlocks, sizeclass, NULL);
+
   return sb;
 }
