@@ -1,10 +1,11 @@
 ///-*-C++-*-//////////////////////////////////////////////////////////////////
 //
-// Hoard: A Fast, Scalable, and Memory-Efficient Allocator
-//        for Shared-Memory Multiprocessors
-// Contact author: Emery Berger, http://www.cs.utexas.edu/users/emery
+// The Hoard Multiprocessor Memory Allocator
+// www.hoard.org
 //
-// Copyright (c) 1998-2000, The University of Texas at Austin.
+// Author: Emery Berger, http://www.cs.utexas.edu/users/emery
+//
+// Copyright (c) 1998-2001, The University of Texas at Austin.
 //
 // This library is free software; you can redistribute it and/or modify
 // it under the terms of the GNU Library General Public License as
@@ -23,7 +24,7 @@
   The superblock class controls a number of blocks (which are
   allocatable units of memory).
   ------------------------------------------------------------------------
-  @(#) $Id: superblock.h,v 1.28 2000/02/11 13:48:02 emery Exp $
+  @(#) $Id: superblock.h,v 1.41 2001/12/20 15:36:20 emery Exp $
   ------------------------------------------------------------------------
   Emery Berger                    | <http://www.cs.utexas.edu/users/emery>
   Department of Computer Sciences |             <http://www.cs.utexas.edu>
@@ -36,32 +37,32 @@
 
 #include "config.h"
 
-#include <assert.h>
 #include <new.h>
-#include <stdio.h>
-#include <stdlib.h>
+//#include <stdio.h>
 
 #include "arch-specific.h"
 #include "block.h"
 
 class hoardHeap; // forward declaration
-
+class processHeap; // forward declaration
 
 class superblock {
 
 public:
 
-  // Construct a superblock for a given size class,
-  // and set the owner to heap o.
+  // Construct a superblock for a given size class and set the heap
+  // owner.
   superblock (int numblocks,
 	      int sizeclass,
 	      hoardHeap * owner);
 
   ~superblock (void)
-    {}
+  {
+	  hoardLockDestroy (_upLock);
+  }
 
   // Make (allocate or re-use) a superblock for a given size class.
-  static superblock * makeSuperblock (int sizeclass);
+  static superblock * makeSuperblock (int sizeclass, processHeap * pHeap);
 
   // Find out who allocated this superblock.
   inline hoardHeap * getOwner (void);
@@ -108,10 +109,19 @@ public:
   // does it have the right magic number?)
   inline int isValid (void);
 
-private:
+  inline void upLock (void) {
+    hoardLock (_upLock);
+  }
+
+  inline void upUnlock (void) {
+    hoardUnlock (_upLock);
+  }
 
   // Compute the 'fullness' of this superblock.
-  inline int computeFullness (void);
+  inline void computeFullness (void);
+
+private:
+
 
   // Disable copying and assignment.
 
@@ -135,6 +145,10 @@ private:
   superblock * 	_next;		// The next superblock in the list.
   superblock * 	_prev;		// The previous superblock in the list.
 
+  bool dirtyFullness;
+
+  hoardLockType _upLock;	// Lock this when moving a superblock to the global (process) heap.
+
   // We insert a cache pad here to prevent false sharing with the
   // first block (which immediately follows the superblock).
 
@@ -144,9 +158,7 @@ private:
 
 hoardHeap * superblock::getOwner (void)
 {
-#if HEAP_DEBUG
-  assert (_magic == SUPERBLOCK_MAGIC);
-#endif
+  assert (isValid());
   hoardHeap * o = _owner;
   return o;
 }
@@ -154,24 +166,19 @@ hoardHeap * superblock::getOwner (void)
 
 void superblock::setOwner (hoardHeap * o) 
 {
-#if HEAP_DEBUG
-  assert (_magic == SUPERBLOCK_MAGIC);
-#endif
-  // Either the current owner is NULL or the new owner will be.
-  assert ((_owner == NULL) || (o == NULL));
+  assert (isValid());
   _owner = o;
 }
 
 
 block * superblock::getBlock (void)
 {
-#if HEAP_DEBUG
-  assert (_magic == SUPERBLOCK_MAGIC);
-#endif
+  assert (isValid());
   // Pop off a block from this superblock's freelist,
   // if there is one available.
   if (_freeList == NULL) {
     // The freelist is empty.
+    assert (getNumAvailable() == 0);
     return NULL;
   }
   assert (getNumAvailable() > 0);
@@ -181,7 +188,8 @@ block * superblock::getBlock (void)
 
   b->setNext(NULL);
 
-  _fullness = computeFullness();
+  dirtyFullness = true;
+  //  computeFullness();
 
   return b;
 }
@@ -189,69 +197,55 @@ block * superblock::getBlock (void)
 
 void superblock::putBlock (block * b)
 {
-#if HEAP_DEBUG
-  assert (_magic == SUPERBLOCK_MAGIC);
-#endif
+  assert (isValid());
   // Push a block onto the superblock's freelist.
-#if HEAP_DEBUG
   assert (b->isValid());
-#endif
   assert (b->getSuperblock() == this);
   assert (getNumAvailable() < getNumBlocks());
   b->setNext (_freeList);
   _freeList = b;
   _numAvailable++;
-  _fullness = computeFullness();
+
+  dirtyFullness = true;
+  //  computeFullness();
 }
 
 int superblock::getNumAvailable (void)
 {
-#if HEAP_DEBUG
-  assert (_magic == SUPERBLOCK_MAGIC);
-#endif
+  assert (isValid());
   return _numAvailable;
 }
 
 
 int superblock::getNumBlocks (void)
 {
-#if HEAP_DEBUG
-  assert (_magic == SUPERBLOCK_MAGIC);
-#endif
+  assert (isValid());
   return _numBlocks;
 }
 
 
 int superblock::getBlockSizeClass (void)
 {
-#if HEAP_DEBUG
-  assert (_magic == SUPERBLOCK_MAGIC);
-#endif
+  assert (isValid());
   return _sizeClass;
 }
 
 
 superblock * const superblock::getNext (void)
 {
-#if HEAP_DEBUG
-  assert (_magic == SUPERBLOCK_MAGIC);
-#endif
+  assert (isValid());
   return _next; 
 }
 
 superblock * const superblock::getPrev (void)
 {
-#if HEAP_DEBUG
-  assert (_magic == SUPERBLOCK_MAGIC);
-#endif
+  assert (isValid());
   return _prev; 
 }
 
 
 void superblock::insertBefore (superblock * nextSb) {
-#if HEAP_DEBUG
-  assert (_magic == SUPERBLOCK_MAGIC);
-#endif
+  assert (isValid());
   // Insert this superblock before the next one (nextSb).
   assert (nextSb != this);
   _next = nextSb;
@@ -263,9 +257,6 @@ void superblock::insertBefore (superblock * nextSb) {
 
 
 void superblock::remove (void) {
-#if HEAP_DEBUG
-  assert (_magic == SUPERBLOCK_MAGIC);
-#endif
   // Remove this superblock from a doubly-linked list.
   if (_next) {
     _next->_prev = _prev;
@@ -280,25 +271,27 @@ void superblock::remove (void) {
 
 int superblock::isValid (void)
 {
-#if HEAP_DEBUG
-  return (_magic == SUPERBLOCK_MAGIC)
-    && (_numBlocks > 0)
-    && (_numAvailable <= _numBlocks)
-    && (_sizeClass >= 0);
-#else
+  assert (_numBlocks > 0);
+  assert (_numAvailable <= _numBlocks);
+  assert (_sizeClass >= 0);
   return 1;
-#endif
 }
 
 
-int superblock::computeFullness (void)
+void superblock::computeFullness (void)
 {
-  return (((SUPERBLOCK_FULLNESS_GROUP - 1)
-	   * (getNumBlocks() - getNumAvailable())) / getNumBlocks());
+  assert (isValid());
+  _fullness = (((SUPERBLOCK_FULLNESS_GROUP - 1)
+		* (getNumBlocks() - getNumAvailable())) / getNumBlocks());
 }
 
 int superblock::getFullness (void)
 {
+  assert (isValid());
+  if (dirtyFullness) {
+    computeFullness();
+    dirtyFullness = false;
+  }
   return _fullness;
 }
 
