@@ -1,10 +1,11 @@
 ///-*-C++-*-//////////////////////////////////////////////////////////////////
 //
-// Hoard: A Fast, Scalable, and Memory-Efficient Allocator
-//        for Shared-Memory Multiprocessors
-// Contact author: Emery Berger, http://www.cs.utexas.edu/users/emery
+// The Hoard Multiprocessor Memory Allocator
+// www.hoard.org
 //
-// Copyright (c) 1998-2000, The University of Texas at Austin.
+// Author: Emery Berger, http://www.cs.utexas.edu/users/emery
+//
+// Copyright (c) 1998-2001, The University of Texas at Austin.
 //
 // This library is free software; you can redistribute it and/or modify
 // it under the terms of the GNU Library General Public License as
@@ -22,7 +23,7 @@
 
 #include "config.h"
 
-#include "heap.h"
+#include "hoardheap.h"
 #include "threadheap.h"
 #include "processheap.h"
 
@@ -49,7 +50,7 @@ void * threadHeap::malloc (const size_t size)
   // We usually have memory locally so we first look for space in the
   // superblock list.
 
-  superblock * sb = findAvailableSuperblock (sizeclass, b);
+  superblock * sb = findAvailableSuperblock (sizeclass, b, _pHeap);
 
   if (sb == NULL) {
     
@@ -62,9 +63,10 @@ void * threadHeap::malloc (const size_t size)
     // If we didn't get any memory from the process heap,
     // we'll have to allocate our own superblock.
     if (sb == NULL) {
-      sb = superblock::makeSuperblock (sizeclass);
+      sb = superblock::makeSuperblock (sizeclass, _pHeap);
       if (sb == NULL) {
 	// We're out of memory!
+	unlock ();
 	return NULL;
       }
 #if HEAP_LOG
@@ -78,17 +80,12 @@ void * threadHeap::malloc (const size_t size)
 #endif
     }
     
-    // Update the stats.
-    incStats (sizeclass,
-	      sb->getNumBlocks() - sb->getNumAvailable(),
-	      sb->getNumBlocks());
-
     // Get a block from the superblock.
     b = sb->getBlock ();
     assert (b != NULL);
 
     // Insert the superblock into our list.
-    insertSuperblock (sizeclass, sb);
+    insertSuperblock (sizeclass, sb, _pHeap);
   }
 
   assert (b != NULL);
@@ -97,19 +94,15 @@ void * threadHeap::malloc (const size_t size)
 
   b->markAllocated();
 
-#if HEAP_FRAG_STATS
-  b->setRequestedSize (align(size));
-  _pHeap->setAllocated (align(size), 0);
-#endif
-
-  // Update the stats.
-  incUStats (sizeclass);
-
 #if HEAP_LOG
   MemoryRequest m;
   m.malloc ((void *) (b + 1), align(size));
   _pHeap->getLog(getIndex()).append(m);
 #endif
+#if HEAP_FRAG_STATS
+  b->setRequestedSize (align(size));
+  _pHeap->setAllocated (align(size), 0);
+#endif
 
   unlock();
   
