@@ -1,10 +1,11 @@
 ///-*-C++-*-//////////////////////////////////////////////////////////////////
 //
-// Hoard: A Fast, Scalable, and Memory-Efficient Allocator
-//        for Shared-Memory Multiprocessors
-// Contact author: Emery Berger, http://www.cs.utexas.edu/users/emery
+// The Hoard Multiprocessor Memory Allocator
+// www.hoard.org
 //
-// Copyright (c) 1998-2000, The University of Texas at Austin.
+// Author: Emery Berger, http://www.cs.utexas.edu/users/emery
+//
+// Copyright (c) 1998-2001, The University of Texas at Austin.
 //
 // This library is free software; you can redistribute it and/or modify
 // it under the terms of the GNU Library General Public License as
@@ -22,9 +23,8 @@
 
 #include "config.h"
 
-#include <string.h>
 
-#include "heap.h"
+#include "hoardheap.h"
 
 class processHeap; // forward declaration
 
@@ -82,6 +82,9 @@ void * threadHeap::memalign (size_t alignment,
     alignment = sizeof(block);
   }
 
+  // Alignment must be a power of two!
+  assert ((alignment & (alignment - 1)) == 0);
+
   // Leave enough room to align the block within the malloced space.
   newSize = size + sizeof(block) + alignment;
 
