@@ -1,10 +1,11 @@
 ///-*-C++-*-//////////////////////////////////////////////////////////////////
 //
-// Hoard: A Fast, Scalable, and Memory-Efficient Allocator
-//        for Shared-Memory Multiprocessors
-// Contact author: Emery Berger, http://www.cs.utexas.edu/users/emery
+// The Hoard Multiprocessor Memory Allocator
+// www.hoard.org
 //
-// Copyright (c) 1998-2000, The University of Texas at Austin.
+// Author: Emery Berger, http://www.cs.utexas.edu/users/emery
+//
+// Copyright (c) 1998-2001, The University of Texas at Austin.
 //
 // This library is free software; you can redistribute it and/or modify
 // it under the terms of the GNU Library General Public License as
@@ -52,10 +53,6 @@
 #include <windows.h>
 #endif
 
-#if defined(__BEOS__)
-#include <OS.h>
-#endif
-
 /*** USAGE: ***
 
    Timer t;
@@ -173,21 +170,6 @@ private:
 #endif // WIN32
 
 
-#ifdef __BEOS__
-#define TIMER_FOUND
-  bigtime_t _starttime, _elapsedtime;
-  bigtime_t _time(void) {
-    return system_time();
-  }
-  double _timetosec (bigtime_t& t) {
-    return (double) t / 1000000.0;
-  }
-  
-  bigtime_t _sectotime (double sec) {
-    return (bigtime_t)(sec * 1000000.0);
-  }
-#endif // __BEOS__
-
 #ifndef TIMER_FOUND
 
   long _starttime, _elapsedtime;
